/***
*
*	Errorsys.prg
*
*  Standard Clipper error handler
*
*  Copyright (c) 1990-1993, Computer Associates International, Inc.
*  All rights reserved.
*
*  Compile:  /m /n /w
*
*/
#include "set.ch"
#include "error.ch"
#include "controls.ch"


// put messages to STDERR
#command ? <list,...>   =>  ?? Chr(13) + Chr(10) ; ?? <list>
#command ?? <list,...>  =>  OutErr(<list>)


// used below
#define NTRIM(n)		( LTrim(Str(n)) )



/***
*	ErrorSys()
*
*	Note:  automatically executes at startup
*/

proc ErrorSys()
	Public l_Controls_Sys
	ErrorBlock( {|e| DefError(e)} )
	l_Controls_Sys := .f.
return

/***
*	DefError()
*/
static func DefError(e)
	local i, cMessage, aOptions, nChoice
	local nHandle, cErrScreen
	local nCount
        local _cDevice  := ""
	local cPrintStr := ""
	local crlf      := chr(13)+chr(10)

	// by default, division by zero yields zero
	if ( e:genCode == EG_ZERODIV )
		return (0)
	end


	// for network open error, set NETERR() and subsystem default
	if ( e:genCode == EG_OPEN .and. e:osCode == 32 .and. e:canDefault )

		NetErr(.t.)
		return (.f.)                             // NOTE

	end


	// for lock error during APPEND BLANK, set NETERR() and subsystem default
	if ( e:genCode == EG_APPENDLOCK .and. e:canDefault )

		NetErr(.t.)
		return (.f.)                             // NOTE

	end



	// build error message
	cMessage := ErrorMessage(e)


	// build options array
	// aOptions := {"Break", "Quit"}
	aOptions := {"Quit"}

	if (e:canRetry)
		AAdd(aOptions, "Retry")
	end

	if (e:canDefault)
		AAdd(aOptions, "Default")
	end


	// put up alert box
	nChoice := 0
        _cDevice := set(_SET_DEVICE)
        set device to screen
        while ( nChoice == 0 )

		if ( Empty(e:osCode) )
			if l_Controls_Sys != .t.
			   nChoice := Alert( cMessage, aOptions )
			else
			   nChoice := Pop_Alert(cMessage, aOptions, e )
			endif
		else
			if l_Controls_Sys != .t.
			   nChoice := Alert( cMessage + ;
			 			";(DOS Error " + NTRIM(e:osCode) + ")", ;
						aOptions )
			else
				nChoice := Pop_Alert(cMessage + ";(DOS Error " + NTRIM(e:osCode) +;
				           ")", aOptions, e)
			endif
		end

		if ( nChoice == NIL )
			exit
		end

	end
        set(_SET_DEVICE,_cDevice)

	if ( !Empty(nChoice) )

		// do as instructed
		if ( aOptions[nChoice] == "Break" )
			Break(e)

		elseif ( aOptions[nChoice] == "Retry" )
			return (.t.)

		elseif ( aOptions[nChoice] == "Default" )
			return (.f.)

		end

	end

        _cDevice := set(_SET_DEVICE)
        set device to screen

	// display message and traceback
	if ( !Empty(e:osCode) )
		cMessage += " (DOS Error " + NTRIM(e:osCode) + ") "
	end

	? cMessage
	i := 2
	while ( !Empty(ProcName(i)) )
		? "Called from", Trim(ProcName(i)) + ;
			"(" + NTRIM(ProcLine(i)) + ")  "

		i++
	end
	if !file("PROG_ERR.TXT")
		nHandle := fcreate("PROG_ERR.TXT",0)
	else
		nHandle := fopen("PROG_ERR.TXT",2)
		fseek(nHandle,0,2)
	endif
	if nHandle > 0
		cErrScreen := savescreen()
		for nCount := 1 to len(cErrScreen) step 2
			cPrintStr := cPrintStr+substr(cErrScreen,nCount,1)
			if len(cPrintStr) == 80
				cPrintStr := cPrintStr+chr(13)+chr(10)
				fwrite(nHandle,cPrintStr)
				cPrintStr := ""
			endif
		next
	endif
	fwrite(nHandle, crlf )
	fwrite(nHandle,"OPEN DATABASE FILES:" + crlf )
	for nCount := 1 to 255
		if !empty(alias(nCount))
			if alias() == alias(nCount)
				fwrite(nHandle, "   Alias: "+padr(alias(nCount),10)  +;
					"  -- Current" + crlf)
			else
				fwrite(nHandle,"   Alias: "+padr(alias(nCount),10) + crlf)
			endif
		endif
	next
	fwrite(nHandle, crlf)
	fwrite(nHandle,    "SYSTEM INFO:" + crlf)
	fwrite(nHandle,    "   Operating System       : "+ OS() + crlf )
	fwrite(nHandle,    "   Free DOS Memory        : "+alltrim(str(memory(0)))+"Kb"+crlf)
	fwrite(nHandle,    "   Free Contiguous Memory : "+alltrim(str(memory(1)))+"Kb"+crlf)
	if !empty(getenv("PATH"))
		fwrite(nHandle,"   SET PATH = "+getenv("PATH") + crlf)
	endif
	if !empty(getenv("CLIPPER"))
		fwrite(nHandle,"   SET CLIPPER = "+getenv("CLIPPER") + crlf)
	endif

	fwrite(nHandle, crlf + "Date: "+dtoc(date())+"  Time: " + Time() + crlf)
	fwrite(nHandle, crlf)

	fclose(nHandle)

	// give up
	if nHandle > 0
		if l_Controls_Sys == .t.
		   msg_box(nil,nil, {"The Error Info was saved to PROG_ERR.TXT"},"ERROR INFORMATION",nil,"W/R")
		else
		   alert("The Error Info was saved to PROG_ERR.TXT.",{"OK"})
		endif
	endif

	set(_SET_DEVICE,_cDevice)

	ErrorLevel(1)
	QUIT

return (.f.)


/***
*	ErrorMessage()
*/
static func ErrorMessage(e)
	local cMessage


	// start error message
	cMessage := if( e:severity > ES_WARNING, "Error ", "Warning " )


	// add subsystem name if available
	if ( ValType(e:subsystem) == "C" )
		cMessage += e:subsystem()
	else
		cMessage += "???"
	end


	// add subsystem's error code if available
	if ( ValType(e:subCode) == "N" )
		cMessage += ("/" + NTRIM(e:subCode))
	else
		cMessage += "/???"
	end


	// add error description if available
	if ( ValType(e:description) == "C" )
		cMessage += ("  " + e:description)
	end


	// add either filename or operation
	if ( !Empty(e:filename) )
		cMessage += (": " + e:filename)

	elseif ( !Empty(e:operation) )
		cMessage += (": " + e:operation)

	end
return (cMessage)

//== Function ==================================================================
//         Name: Pop_Alert()           Docs: Raymond T. Deal
//  Description:
//       Author: Raymond T. Deal
// Date created: 11-03-94              Date updated: 11-03-94
// Time created: 02:28:23pm            Time updated: 02:28:23pm
//    Copyright: Raymond T. Deal
//------------------------------------------------------------------------------
//    Arguments: cMessage
//             : aOptions
// Return Value: nReturn
//     See Also:
//==============================================================================
Function Pop_Alert(cMessage, aOptions, e)
	Local GetList  := {}
	Local nStrLen  := len(cMessage)+4
	Local nCount   := 1
	Local nLen     :=  0
	Local cHeader  := "ERROR MESSAGE"
	Local cColor   := "W/R"
	Local nT       := int((maxrow()+1-6)/2)
	Local nB       := 7
	Local nL       := 0
	Local nR       := 0
	Local nButLen  := 0
	Local nButLeft := 0
	Local nSkip    :=  0
	Local bBlock   := nil
	Local nReturn  := 1

	aadd(aOptions, "VIEW")
	for nCount := 1 to len(aOptions)
		nLen := nLen+len(aOptions[nCount])+5
		nButLen := nButLen+len(aOptions[nCount])+5
	next

	nLen := if(nStrLen > nLen, nStrLen, nLen+4)
	nLen := if(nLen < 20, 20, nLen)

	nL := int((maxcol()+1-nLen)/2)
	nR := nLen

	@ nT, nL Get as DialogBox        ;
		Header cHeader               ;           // Header for the Dialog Box
		Height nB                    ;           // Height of the Dialog Box
		Length nR                    ;           // Length of the Dialog Box
		Color cColor

	@ 2,2 Say Text cMessage Color cColor

	nButLeft := int((nLen-nButLen)/2)

	for nCount := 1 to len(aOptions)
		if aOptions[nCount] != "VIEW"
			bBlock := {|oGet| oGet:exitstate := GE_ESCAPE, nReturn := Get_Focus_Get(GetList) }
		else
			bBlock := {|oGet| View_Error(e), oGet:exitstate := GE_NOEXIT }
		endif

		@ 4,nButLeft+nSkip get as pushbutton    ;
			Prompt aOptions[nCount]             ;
			Color cColor                        ;
			Action bBlock

		nSkip := nSkip+len(aOptions[nCount])+5
	next

	read
	kill dialogbox

Return nReturn

//== Function ==================================================================
//         Name: Get_Focus_Get()       Docs: Raymond T. Deal
//  Description:
//       Author: Raymond T. Deal
// Date created: 11-03-94              Date updated: 11-03-94
// Time created: 02:28:16pm            Time updated: 02:28:16pm
//    Copyright: Raymond T. Deal
//------------------------------------------------------------------------------
//    Arguments: GetList
// Return Value: nReturn
//     See Also:
//==============================================================================
Static Function Get_Focus_Get(GetList)
	Local nCount := 1
	Local nReturn := 0

	for nCount := 1 to len(getlist)
		if getlist[nCount]:hasfocus
			nReturn := nCount
		endif
	next
Return nReturn

//== Function ==================================================================
//         Name: View_Error()          Docs: Raymond T. Deal
//  Description:
//       Author: Raymond T. Deal
// Date created: 11-03-94              Date updated: 11-03-94
// Time created: 04:22:50pm            Time updated: 04:22:50pm
//    Copyright: Raymond T. Deal
//------------------------------------------------------------------------------
//    Arguments: e
// Return Value: nil
//     See Also:
//==============================================================================
Function View_Error(e)
	Local GetList := {}
	Local cHeader := "ERROR VIEWER"
	Local cColor  := "N/BG"
	Local nTop    := int((maxrow()-16)/2)
	Local nItem   := 2
	Local aProcLine := {}
	Local cLineName := padl(alltrim(str(procline(nItem))),5)+"  "+padr(alltrim(procname()),10)
	Local oBrowse := TBrowseNew(0,0,0,0)
	Local oColumn := TBColumnNew()
	Local nElem := 1

	while ( !Empty(ProcName(nItem)) )
		aadd(aProcLine, padl(alltrim(str(procline(nItem))),5)+"  "+padr(alltrim(procname(nItem)),10))
		nItem++
	end
	oColumn:block := {|| aProcLine[nElem]+" " }
	oBrowse:addcolumn(oColumn)
	oBrowse:skipblock := {|nHowMany| ArraySkip(nHowMany,@nElem,aProcLine) }
	oBrowse:gobottomblock := {|| nElem := len(aProcLine) }
	oBrowse:gotopBlock    := {|| nElem := 1 }

	@ nTop, 4  Get as DialogBox      ;
		Header cHeader               ;           // Header for the Dialog Box
		Height 16                    ;           // Height of the Dialog Box
		Length 70                    ;           // Length of the Dialog Box
		Color cColor

	@ 2,3  Say Text "Error:Args      : "+ Error2Str(e:args) color cColor
	@ 2,35 Say Text "Error:CanDefault: "+ Error2Str(e:candefault) color cColor
	@ 3,3  Say Text "Error:CanRetry  : "+ Error2Str(e:canRetry)  color cColor
	@ 3,35 Say Text "Error:CanSub    : "+ Error2Str(e:canSubstitute) color cColor
	@ 4,3  Say Text "Error:Cargo     : "+ Error2Str(e:cargo) color cColor
	@ 4,35 Say Text "Error:Tries     : "+ Error2Str(e:tries) color cColor
	@ 5,3  Say Text "Error:GenCode   : "+ Error2Str(e:genCode) color cColor
	@ 5,35 Say Text "Error:OSCode    : "+ Error2Str(e:osCode) color cColor
	@ 6,3  Say Text "Error:SubCode   : "+ Error2Str(e:subcode) color cColor
	@ 6,35 Say Text "Error:SubSystem : "+ Error2Str(e:subsystem) color cColor
	@ 7,3  Say Text "Error:Operation : "+ Error2Str(e:operation) color cColor
	@ 8,3  Say Text "Error:Descript  : "+ Error2Str(e:description) color cColor
	@ 9,3  Say Text "Error:Filename  : "+ left(Error2Str(e:filename),23) color cColor
	@ 10,3 Say Text "Error:Severity  : "+ Error2Str(e:severity) color cColor

	@ 8,44 Say Text "  Proc Line/Name" color cColor

	@ 9,46 Get as BrowseBox           ;
		Prompt ""                     ;
		Browse oBrowse                ;
		Height 6                      ;
		Length 20                     ;
		Color cColor

	//@ 11,2 Get cLineName as Dropdown  ;
	//	   Prompt "Proc Line & Name :";
	//	   array aProcLine            ;
	//	   DropDown Width 18          ;
	//	   DropDown Height 5          ;
	//	   Color cColor

	@ 13, 32 Get as Pushbutton        ;
		Prompt "~OK"                  ;
		Color cColor                  ;
		Hotkey K_ALT_O                ;
		Action {|oGet| oGet:exitstate := GE_ESCAPE }

	Read

	Kill DialogBox

Return nil

//== Function ==================================================================
//         Name: Error2Str()           Docs: Raymond T. Deal
//  Description:
//       Author: Raymond T. Deal
// Date created: 11-03-94              Date updated: 11-03-94
// Time created: 04:22:46pm            Time updated: 04:22:46pm
//    Copyright: Raymond T. Deal
//------------------------------------------------------------------------------
//    Arguments: x
// Return Value: cReturn
//     See Also:
//==============================================================================
Function Error2Str(x)
	Local cReturn := ""

	if valtype(x) == "N"
		cReturn := alltrim(str(x))
	elseif valtype(x) == "C"
		cReturn := alltrim(x)
	elseif valtype(x) == "L"
		cReturn := if(x, "YES","NO")
	elseif valtype(x) == "D"
		cReturn := dtoc(x)
	elseif valtype(x) == "O"
		cReturn := "Object"
	elseif valtype(x) == "A"
		cReturn := "Array"
	elseif valtype(x) == "U"
		cReturn := "NIL"
	elseif valtype(x) == "B"
		cReturn := "Code Block"
	else
		cReturn := "Undefined"
	endif

Return cReturn


//== Function ==================================================================
//         Name: ArraySkip()           Docs: Raymond T. Deal
//  Description:
//       Author: Raymond T. Deal
// Date created: 10-05-94              Date updated: 10-05-94
// Time created: 10:47:21am            Time updated: 10:47:21am
//    Copyright: Raymond T. Deal
//------------------------------------------------------------------------------
//    Arguments: nHowMany
//             : nCurPos
//             : aArray
// Return Value: nActual
//     See Also:
//==============================================================================
Function ArraySkip(nHowMany,nCurPos,aArray)      // My Array Skipper
	Local nLen    := len(aArray)
	Local nActual := 0

	If nHowMany >= 0
		If (nCurPos+nHowMany) > nLen
			nActual := nLen-nCurPos
			nCurPos := nLen
		Else
			nActual := nHowMany
			nCurPos += nHowMany
		Endif
	Else
		IF (nCurPos+nHowMany) < 1
			nActual := 1-nCurPos
			nCurPos := 1
		Else
			nActual := nHowMany
			nCurPos += nHowMany
		Endif
	Endif

Return(nActual)
